from data_inclusion.schema.base import EnhancedEnum


class ModeMobilisation(EnhancedEnum):
    """Modes de mobilisation"""

    ENVOYER_UN_COURRIEL = (
        "envoyer-un-courriel",
        "Envoyer un courriel",
        (
            "Envoyer un courriel à l’adresse électronique associée au service, ou à"
            " défaut, à la structure."
        ),
    )
    SE_PRESENTER = (
        "se-presenter",
        "Se présenter",
        (
            "Se présenter directement à l’adresse associée au service ou, à défaut, à"
            " la structure."
        ),
    )
    TELEPHONER = (
        "telephoner",
        "Téléphoner",
        "Téléphoner au numéro associé au service, ou à défaut, à la structure.",
    )
    UTILISER_LIEN_MOBILISATION = (
        "utiliser-lien-mobilisation",
        "Utiliser le lien de mobilisation",
        "Utiliser le lien de mobilisation du service.",
    )


class PersonneMobilisatrice(EnhancedEnum):
    """Personnes mobilisatrices"""

    USAGERS = (
        "usagers",
        "Usagers",
        "Service mobilisable par les usagers.",
    )
    PROFESSIONNELS = (
        "professionnels",
        "Professionnels",
        "Service mobilisable par les professionnels de l’insertion.",
    )



from typing import Callable, Mapping

import pendulum

from data_inclusion.schema.v1 import (
    Frais,
    ModeAccueil,
    ModeMobilisation,
    Service,
)


def adresse_bien_definie(service: Service) -> float | None:
    critere_applicable = (
        service.modes_accueil and ModeAccueil.EN_PRESENTIEL in service.modes_accueil
    ) or (
        service.modes_mobilisation
        and ModeMobilisation.SE_PRESENTER in service.modes_mobilisation
    )

    if critere_applicable:
        return 1.0 if service.adresse else 0.0

    return None


def telephone_bien_defini(service: Service) -> float | None:
    critere_applicable = (
        service.modes_mobilisation
        and ModeMobilisation.TELEPHONER in service.modes_mobilisation
    )
    if critere_applicable:
        return 1.0 if service.telephone else 0.0

    return None


def courriel_bien_defini(service: Service) -> float | None:
    critere_applicable = (
        service.modes_mobilisation
        and ModeMobilisation.ENVOYER_UN_COURRIEL in service.modes_mobilisation
    )

    if critere_applicable:
        return 1.0 if service.courriel else 0.0

    return None


def modes_mobilisation_bien_definis(service: Service) -> float:
    match service:
        case Service(modes_mobilisation=None, mobilisable_par=None):
            return 0.0
        case Service(modes_mobilisation=modes_mobilisation, mobilisable_par=None) if (
            modes_mobilisation is not None
        ):
            return 0.5
        case Service(modes_mobilisation=None, mobilisable_par=mobilisable_par) if (
            mobilisable_par is not None
        ):
            return 0.5
        case Service(
            modes_mobilisation=modes_mobilisation, mobilisable_par=mobilisable_par
        ) if modes_mobilisation is not None and mobilisable_par is not None:
            return 1.0
        case _:
            raise ValueError()


def date_maj_recente(service: Service) -> float | None:
    SIX_MOIS = pendulum.duration(months=6)
    DEUX_ANS = pendulum.duration(years=2)

    if service.source in ["agefiph", "france-travail"]:
        return None

    if service.date_maj is None:
        return 0.0

    age = pendulum.instance(service.date_maj).diff(pendulum.now())

    if age < SIX_MOIS:
        return 1.0
    elif SIX_MOIS < age < DEUX_ANS:
        # simple interpolation linéaire entre 0 et 1
        return (DEUX_ANS - age) / (DEUX_ANS - SIX_MOIS)
    else:
        return 0.0


def au_moins_une_thematique(service: Service) -> float:
    return 1.0 if service.thematiques else 0.0


def au_moins_un_public(service: Service) -> float:
    return 1.0 if service.publics else 0.0


def au_moins_un_moyen_de_contact(service: Service) -> float:
    return (
        1.0
        if any(
            (
                service.courriel,
                service.lien_mobilisation,
                service.telephone,
            )
        )
        else 0.0
    )


def description_bien_definie(service: Service) -> float:
    SEUIL_MINIMUM = 200
    SEUIL_BON = 400

    if len(service.description) < SEUIL_MINIMUM:
        return 0.0
    elif len(service.description) < SEUIL_BON:
        # simple interpolation linéaire entre 0 et 1
        return (len(service.description) - SEUIL_MINIMUM) / (SEUIL_BON - SEUIL_MINIMUM)
    else:
        return 1.0


def frais_bien_definis(service: Service) -> float:
    match service:
        case Service(frais=None):
            return 0.0
        case Service(frais=Frais.PAYANT, frais_precisions=None):
            return 0.5
        case Service(frais=Frais.PAYANT, frais_precisions=frais_precisions) if (
            frais_precisions
        ):
            return 1.0
        case Service(frais=Frais.GRATUIT):
            return 1.0
        case _:
            raise ValueError()


# Les critères sont implémentés sous forme de fonctions qui prennent un service en
# entrée et renvoient un score entre 0 et 1.
# Dans le cas où le critère n'est pas applicable, une fonction peut renvoyer None.
# Par exemple, le critère "adresse_bien_definie" ne s'applique que si le service propose
# un mode d'accueil en présentiel ou une orientation bénéficiaire en présentiel.
CritereFn = Callable[[Service], float | None]

CRITERES: list[CritereFn] = [
    adresse_bien_definie,
    modes_mobilisation_bien_definis,
    au_moins_un_public,
    au_moins_une_thematique,
    au_moins_un_moyen_de_contact,
    courriel_bien_defini,
    date_maj_recente,
    frais_bien_definis,
    description_bien_definie,
    telephone_bien_defini,
]


def score(service: Service) -> tuple[float, Mapping[str, float | None]]:
    resultats = {critere.__name__: critere(service) for critere in CRITERES}

    resultats_pertinents = {
        k: round(v, 2) for k, v in resultats.items() if v is not None
    }
    score = sum([v for v in resultats_pertinents.values()]) / len(resultats_pertinents)

    return round(score, 2), resultats



from data_inclusion.schema.base import EnhancedEnum


class TypeService(EnhancedEnum):
    """Types de services"""

    ACCOMPAGNEMENT = (
        "accompagnement",
        "Accompagnement",
        "Accompagnement global pour le retour à l’emploi",
    )
    AIDE_FINANCIERE = (
        "aide-financiere",
        "Aide financière",
        "Prise en charge financière, aide ou rémunération soutenant le "
        "retour à l’emploi",
    )
    AIDE_MATERIELLE = (
        "aide-materielle",
        "Aide materielle",
        "Accéder à du matériel soutenant le retour à l’emploi (voiture, vélo, "
        "ordinateur, alimentation, etc)",
    )
    ATELIER = (
        "atelier",
        "Atelier",
        "Groupe de travail constitué autour d’une activité, d’un thème "
        "soutenant le retour à l’emploi",
    )
    FORMATION = (
        "formation",
        "Formation",
        "Formation dispensée par des professionnels pour les bénéficiaires",
    )
    INFORMATION = (
        "information",
        "Information",
        "Premier niveau d’information apporté par des professionnels du secteur",
    )



from datetime import date
from typing import Annotated, Literal, Optional

from pydantic import EmailStr, HttpUrl

from data_inclusion.schema import common
from data_inclusion.schema.base import BaseModel, Field
from data_inclusion.schema.v1 import (
    Frais,
    ModeAccueil,
    ModeMobilisation,
    PersonneMobilisatrice,
    Public,
    Thematique,
    TypeService,
)


class Service(BaseModel):
    #####################
    ### Champs requis ###
    #####################
    source: str
    structure_id: str
    id: str
    nom: Annotated[
        str,
        Field(
            description="""
                Nom du service.

                Chaîne de caractères entre 3 et 150 caractères,
                ne se terminant pas par un point.
            """,
            examples=["Atelier insertion et posture professionnelle"],
            min_length=3,
            max_length=150,
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="""
                Description du service.

                Entre 50 et 2000 caractères.

                Ce champ est pris en compte dans le calcul du score de qualité.
            """,
            examples=[
                """
                    Cet atelier-conseil vous permet d’identifier les compétences à
                    développer pour atteindre vos objectifs d’évolution professionnelle
                    et à découvrir les différentes modalités de formation.

                    Durée d’une journée et inscription via votre espace France Travail.
                """
            ],
            min_length=50,
            max_length=2000,
        ),
    ]
    date_maj: Annotated[
        date,
        Field(
            description="""
                Date de dernière modification du service chez le producteur de données.
            """,
            examples=["2025-02-14"],
            title="Date de dernière modification",
        ),
    ]

    #########################
    ### Champs optionnels ###
    #########################
    type: Annotated[
        Optional[TypeService],
        Field(
            description="""
            Type de service.
            """,
            examples=[TypeService.ACCOMPAGNEMENT],
        ),
    ] = None
    thematiques: Optional[set[Thematique]] = None
    frais: Annotated[
        Optional[Frais],
        Field(
            description="""
                Indique si l’accès au service est payant ou gratuit.

                Si le service comporte des frais, ceux-ci devraient être précisés dans
                le champ `frais_precisions`.
            """,
            examples=[Frais.GRATUIT, Frais.PAYANT],
        ),
    ] = None
    frais_precisions: Annotated[
        Optional[str],
        Field(
            title="Précisions sur les frais",
            description="""
                Précisions sur les éventuels frais pour accéder au service.
            """,
            examples=[
                "10€ pour l’adhésion annuelle",
                "Tarif réduit pour les bénéficiaires du RSA",
            ],
        ),
    ] = None
    publics: Annotated[
        Optional[set[Public]],
        Field(
            title="Publics",
            description="""
            Publics visés par le service.

            Des informations complémentaires peuvent être précisées dans le champ
            `publics_precisions`.
        """,
            examples=[Public.FEMMES, Public.RESIDENTS_QPV_FRR],
            min_length=1,
        ),
    ] = None
    publics_precisions: Annotated[
        Optional[str],
        Field(
            title="Précisions sur les publics",
            description="""
            Précisions sur les publics visés par le service.
        """,
            examples=["Le jeune entre 15 et 18 ans."],
        ),
    ] = None
    conditions_acces: Annotated[
        Optional[str],
        Field(
            description="""
            Conditions d’accès au service.

            Il peut s’agir de prérequis ou de justificatifs à présenter.
        """,
            examples=["Maîtrise de la langue française à l’oral et à l’écrit"],
            title="Conditions d’accès",
        ),
    ] = None
    commune: Optional[str] = None
    code_postal: Optional[common.CodePostal] = None
    code_insee: Optional[common.CodeCommune] = None
    adresse: Optional[str] = None
    complement_adresse: Optional[str] = None
    longitude: Optional[float] = None
    latitude: Optional[float] = None
    telephone: Annotated[
        Optional[str],
        Field(
            description="""
                Numéro de téléphone à utiliser pour obtenir des informations
                complémentaires sur le service. Si le mode de mobilisation est
                `telephoner`, peut être utilisé pour mobiliser le service.

                Chaîne de caractères contenant un seul numéro de téléphone,
                de préférence au format E.164.
            """,
            examples=["+33123456789"],
            default=None,
            title="Téléphone",
        ),
    ]
    courriel: Annotated[
        Optional[EmailStr],
        Field(
            description="""
                Courriel à utiliser pour obtenir des informations complémentaires sur
                le service. Si le mode de mobilisation est `envoyer-un-email`, peut
                être utilisé pour mobiliser le service.

                Doit suivre le format de la RFC 5322.
                Vérification de l’existence du destinataire (envoi d’un courrier
                de notification)

                Si non conforme ou destinataire inexistant, suppression de la valeur.
            """,
            examples=["exemple@inclusion.gouv.fr"],
            default=None,
        ),
    ]
    modes_accueil: Optional[set[ModeAccueil]] = None
    zone_eligibilite: Annotated[
        Optional[
            list[
                common.CodeCommune
                | common.CodeDepartement
                | common.CodeEPCI
                | common.CodePays
                | Literal["france"]
                | Literal["99100"]  # Code pays INSEE pour la France métropolitaine
            ]
        ],
        Field(
            title="Zone d’éligibilité",
            min_length=1,
            description="""
            Zone géographique d’éligibilité du service.

            Contient une liste de codes issus du [Code Officiel Géographique](https://www.insee.fr/fr/information/2560452)
            maintenu par l’INSEE.

            Chaque code dans cette liste peut être un code commune, un code département,
            un code EPCI ou un code pays.

            Si le service est éligible à l’ensemble d’une région, lister les codes des
            departements de cette région.

            Si le service est éligible sur l’ensemble du territoire national, utiliser
            le code `france` (France) ou le code pays `99100`.

            data·inclusion vérifie la validité des codes fournis. Les codes invalides
            sont supprimés de la liste.

            [Outil de recherche des codes](https://www.insee.fr/fr/recherche/recherche-geographique)
        """,  # noqa: E501
            examples=[
                ["75056"],
                ["2A", "2B"],
                ["200093201"],
                ["2A", "2B", "200093201"],
                ["france"],
            ],
        ),
    ] = None
    contact_nom_prenom: Optional[str] = None
    lien_mobilisation: Annotated[
        Optional[HttpUrl],
        Field(
            description="""
                Lien pour accéder ou mobiliser l’offre de service.
            """,
            examples=["https://www.actionlogement.fr/demande-cfi"],
        ),
    ] = None
    modes_mobilisation: Annotated[
        Optional[set[ModeMobilisation]],
        Field(
            description="""
                Modes de mobilisation de l’offre de service.
            """,
            examples=[
                "envoyer-un-courriel",
            ],
            min_items=1,
        ),
    ] = None
    mobilisable_par: Annotated[
        Optional[set[PersonneMobilisatrice]],
        Field(
            description="""
                Indique qui peut mobiliser le service : usagers, professionnels ou les
                deux.
            """,
            examples=[
                ["professionnels"],
                ["usagers"],
            ],
            min_items=1,
        ),
    ] = None
    mobilisation_precisions: Annotated[
        Optional[str],
        Field(
            description="""
                Précisions sur les modes de mobilisation du service.
            """,
            examples=[
                """
                    La demande est à faire depuis l’espace personnel
                    du demandeur d’emploi, rubrique « mes aides »,
                    formulaire spécifique « Aide à la mobilité ».
                """
            ],
        ),
    ] = None
    volume_horaire_hebdomadaire: Annotated[
        Optional[float],
        Field(
            description="""
                Durée du service en heures sur une semaine.

                L’absence de valeur signifie que le service n’est pas concerné
                (exemple aide financière) ou que l’information n’est pas
                disponible.

                Champ utilisé dans le cadre des 15h-20h d’activité
                hebdomadaire des BRSA.
            """,
            examples=[1],
            ge=0,
        ),
    ] = None
    nombre_semaines: Annotated[
        Optional[int],
        Field(
            description="""
                Nombre de semaines sur lequel dure le service.

                Ne peut pas être inférieur à 1. L’absence de valeur
                signifie que le service n’est pas concerné (exemple aide financière)
                ou que l’information n’est pas disponible.

                Champ utilisé dans le cadre des 15h-20h d’activité hebdomadaire
                des BRSA.
            """,
            examples=[3],
            ge=1,
        ),
    ] = None
    horaires_accueil: Annotated[
        Optional[str],
        Field(
            description="""
            Horaires d’accueil du public pour ce service.

            Si le champ n’est pas renseigné, les horaires d’accueil de la structure
            peuvent être utilisés.

            Doit être au format [OpenStreetMap Opening Hours](https://wiki.openstreetmap.org/wiki/FR:Key:opening_hours).

            [Outil d’aide à la saisie](https://projets.pavie.info/yohours/).
        """,
            title="Horaires d’accueil du public",
            examples=["Mo-Fr 08:30-12:30; PH off"],
        ),
    ] = None



from datetime import date
from typing import Annotated, Optional

from pydantic import EmailStr, HttpUrl

from data_inclusion.schema import common
from data_inclusion.schema.base import BaseModel, Field
from data_inclusion.schema.v0 import LabelNational, TypologieStructure


class Structure(BaseModel):
    #####################
    ### Champs requis ###
    #####################
    source: str
    id: str
    nom: Annotated[
        str,
        Field(
            description="""
                Nom de la structure.

                Chaîne de caractères entre 3 et 150 caractères,
                ne se terminant pas par un point.
            """,
            examples=["Centre social Le Tournesol"],
            min_length=3,
            max_length=150,
        ),
    ]
    description: Annotated[
        str,
        Field(
            description="""
                Description de la structure.

                Entre 50 et 2000 caractères.
            """,
            examples=[
                """
                    L’association 3027 offre un accès gratuit aux arts, à la culture et
                    au sport pour toutes et tous sans distinction et en priorité aux
                    personnes en situation de précarité et d’isolement.
                """
            ],
            min_length=50,
            max_length=2000,
        ),
    ]
    date_maj: Annotated[
        date,
        Field(
            description="""
                Date de dernière modification de la
                structure chez le producteur de données.
            """,
            examples=["2025-02-14"],
            title="Date de dernière modification",
        ),
    ]

    #########################
    ### Champs optionnels ###
    #########################
    siret: Optional[common.CodeSiret] = None
    commune: Optional[str] = None
    code_postal: Optional[common.CodePostal] = None
    code_insee: Optional[common.CodeCommune] = None
    adresse: Optional[str] = None
    complement_adresse: Optional[str] = None
    longitude: Optional[float] = None
    latitude: Optional[float] = None
    typologie: Optional[TypologieStructure] = None
    telephone: Annotated[
        Optional[str],
        Field(
            description="""
                Numéro de téléphone à utiliser pour obtenir des informations
                complémentaires sur la structure.

                Chaîne de caractères contenant un seul numéro de téléphone,
                de préfèrence au format E.164.
            """,
            examples=["+33123456789"],
            default=None,
            title="Téléphone",
        ),
    ]
    courriel: Annotated[
        Optional[EmailStr],
        Field(
            description="""
                Courriel à utiliser pour obtenir des informations complémentaires sur
                la structure.

                Doit suivre le format de la RFC 5322.
            """,
            default=None,
            examples=["exemple@inclusion.gouv.fr"],
        ),
    ]
    site_web: Optional[HttpUrl] = None
    horaires_accueil: Annotated[
        Optional[str],
        Field(
            description="""
            Horaires d’accueil du public par la structure.

            Un service peut avoir un horaire d’accueil différent. Se référer aux
            horaires des services.

            Doit être au format [OpenStreetMap Opening Hours](https://wiki.openstreetmap.org/wiki/FR:Key:opening_hours).

            [Outil d’aide à la saisie](https://projets.pavie.info/yohours/).
        """,  # noqa: E501
            title="Horaires d’accueil du public",
            examples=["Mo-Fr 08:30-12:30; PH off"],
        ),
    ] = None
    accessibilite_lieu: Annotated[
        Optional[HttpUrl],
        Field(
            title="Accessibilité du lieu",
            description="""
            Lien vers la page Accesslibre référençant le niveau d’accessibilité de
            la structure.

            Le format attendu est donc un lien vers [Accesslibre](https://acceslibre.beta.gouv.fr/).
            """,
            examples=[
                "https://acceslibre.beta.gouv.fr/app/17-la-greve-sur-mignon/a/mairie/erp/mairie-la-greve-sur-mignon/",  # noqa: E501
            ],
        ),
    ] = None
    labels_nationaux: Optional[set[LabelNational]] = None
    labels_autres: Optional[set[str]] = None



from data_inclusion.schema.base import EnhancedEnum


class Frais(EnhancedEnum):
    """Frais"""

    GRATUIT = (
        "gratuit",
        "Gratuit",
        "L’accès au lieu et/ou à ses services est gratuit",
    )
    PAYANT = (
        "payant",
        "Payant",
        "L’accès au lieu et/ou à ses services est payant",
    )



from data_inclusion.schema.common import (
    CodeCommune,
    CodeDepartement,
    CodeEPCI,
    CodePostal,
    CodeRegion,
    CodeSiren,
    CodeSiret,
)
from data_inclusion.schema.v0.labels_nationaux import LabelNational
from data_inclusion.schema.v0.modes_accueil import ModeAccueil
from data_inclusion.schema.v0.thematiques import Thematique
from data_inclusion.schema.v0.typologies_de_structures import TypologieStructure
from data_inclusion.schema.v0.zones_de_diffusion import ZoneDiffusionType
from data_inclusion.schema.v1.frais import Frais
from data_inclusion.schema.v1.modes_mobilisation import (
    ModeMobilisation,
    PersonneMobilisatrice,
)
from data_inclusion.schema.v1.publics import Public
from data_inclusion.schema.v1.types_de_services import TypeService

# Keep those import last to avoid circular imports
from data_inclusion.schema.v1.services import Service  # isort: skip
from data_inclusion.schema.v1.structures import Structure  # isort: skip

__all__ = [
    "CodeCommune",
    "CodeDepartement",
    "CodeEPCI",
    "CodePostal",
    "CodeRegion",
    "CodeSiren",
    "CodeSiret",
    "Frais",
    "LabelNational",
    "ModeAccueil",
    "ModeMobilisation",
    "PersonneMobilisatrice",
    "Public",
    "Service",
    "Structure",
    "Thematique",
    "TypeService",
    "TypologieStructure",
    "ZoneDiffusionType",
]



from data_inclusion.schema.base import EnhancedEnum


class Public(EnhancedEnum):
    """Publics"""

    ACTIFS = (
        "actifs",
        "Actifs",
        "Toute personne ayant un emploi, qu’il soit salarié, libéral ou à son compte",
    )
    BENEFICIAIRES_DES_MINIMAS_SOCIAUX = (
        "beneficiaires-des-minimas-sociaux",
        "Bénéficiaires des minimas sociaux",
        (
            "Personnes touchant des aides financières telles que RSA, SMIC, allocations"
            " familiales, bourses, etc"
        ),
    )
    DEMANDEURS_EMPLOI = (
        "demandeurs-emploi",
        "Demandeurs d’emploi",
        "Demandeurs d’emploi de tout type",
    )
    ETUDIANTS = (
        "etudiants",
        "Étudiants",
        "Étudiants",
    )
    FAMILLES = (
        "familles",
        "Familles",
        "Familles, parents, enfants, etc",
    )
    FEMMES = (
        "femmes",
        "Femmes",
        "Femmes isolées, victimes de violence, etc",
    )
    JEUNES = (
        "jeunes",
        "Jeunes",
        "Jeunes au sens large, jeunes adultes, adolescents, etc",
    )
    PERSONNES_EN_SITUATION_DE_HANDICAP = (
        "personnes-en-situation-de-handicap",
        "Personnes en situation de handicap",
        "Personnes en situation de handicap de tout type",
    )
    PERSONNES_EN_SITUATION_D_URGENCE = (
        "personnes-en-situation-durgence",
        "Personnes en situation d’urgence",
        "SDF, victimes de violence, grande précarité, etc",
    )
    PERSONNES_EN_SITUATION_JURIDIQUE_SPECIFIQUE = (
        "personnes-en-situation-juridique-specifique",
        "Personnes en situation juridique spécifique",
        "Personnes sortant de détention, détenus ou autre statut juridique spécifique",
    )
    PERSONNES_EXILEES = (
        "personnes-exilees",
        "Personnes exilées",
        "Réfugiés, demandeurs d’asile, personnes immigrées, etc",
    )
    RESIDENTS_QPV_FRR = (
        "residents-qpv-frr",
        "Résidents en QPV ou FRR",
        (
            "Personnes résidentes en zone Quartier Prioritaire de la Ville ou France"
            " Ruralités Revitalisation (anciennement ZRR)"
        ),
    )
    SENIORS = (
        "seniors",
        "Séniors",
        "Personnes âgées, retraités, etc",
    )
